/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: MailBCExternalListener.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.mail.listeners;

import java.io.IOException;
import java.io.InputStream;
import java.util.Timer;
import java.util.logging.Level;

import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Flags.Flag;

import org.ow2.petals.bc.mail.MailBCImpl;
import org.ow2.petals.bc.mail.MissingElementException;
import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class is used to manage listeners for external mail addresses registered
 * during SU deployments
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCExternalListener extends AbstractExternalListener {

    protected ConfigurationExtensions extensions;

    protected MailSessionManager mailSessionManager;

    protected MimeMessageManager mimeMessageManager;

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    private Timer t;

    @Override
    public void init() {
        this.mailSessionManager = ((MailBCImpl) this.getComponent()).getMailSessionManager();
        this.sessionDescriptorBuilder = ((MailBCImpl) this.getComponent())
                .getSessionDescriptorBuilder();
        this.mimeMessageManager = ((MailBCImpl) this.getComponent()).getMimeMessageManager();
    }

    /**
     * Send the message to the jbi service defined in the Consume node
     * 
     * @param message
     */
    protected void process(Message message) {

        this.getLogger().info("process an incoming email");

        Exchange exchange = null;

        try {
            exchange = this.createConsumeExchange(this.getConsumes(), MEPConstants.IN_ONLY_PATTERN);
            this.setJBIContentAndAttachments(message, exchange);
            this.setProperties(message, exchange);
        } catch (final javax.jbi.messaging.MessagingException e) {
            this.getLogger().log(Level.WARNING, "error with JBI Exchange creation", e);
        } catch (final PEtALSCDKException e) {
            this.getLogger().log(Level.WARNING, "error with XML Normalized message creation", e);
        } catch (final MessagingException e) {
            this.getLogger().log(Level.WARNING, "error with email access", e);
        } catch (final IOException e) {
            this.getLogger().log(Level.WARNING, "error with email access", e);
        }
        try {
            this.send(exchange);
        } catch (final javax.jbi.messaging.MessagingException e) {
            this.getLogger().log(Level.WARNING, "the JBI Exchange can not be sent", e);
            try {
                message.setFlag(Flag.DELETED, false);
            } catch (final MessagingException e1) {
                this.getLogger().log(Level.WARNING, "error with email access", e);
            }
        }
    }

    /**
     * set the body and attachments from the mail {@link Part} to the jbi
     * exchange. If the given message {@link Part} is a {@link Multipart}, all
     * sub part are recursively processed.
     * 
     * @param message
     *            the mail {@link Part} to process
     * @param exchange
     *            the resulting jbi exchange
     * @throws MessagingException
     * @throws IOException
     * @throws javax.jbi.messaging.MessagingException
     * @throws PEtALSCDKException
     */
    protected void setJBIContentAndAttachments(Part message, Exchange exchange) throws IOException,
            MessagingException, javax.jbi.messaging.MessagingException, PEtALSCDKException {
        final Object content = message.getContent();
        if (content instanceof String) {
            if (exchange.getInMessageContentAsSource() == null) {
                this.getLogger().info("a text part is set to JBI Exchange content");
                final Document doc = XMLUtil.createDocument();
                final Element bodyElmt = doc.createElement("body");
                bodyElmt.setTextContent(content.toString());
                doc.appendChild(bodyElmt);
                doc.normalize();
                exchange.setInMessageContent(doc);
            } else {
                this.getLogger().warning(
                        "a text part is found, but the JBI Exchange content has been already set");
            }
        } else if (content instanceof Multipart) {
            final Multipart mp = (Multipart) content;
            final int count = mp.getCount();
            for (int i = 0; i < count; i++) {
                this.setJBIContentAndAttachments(mp.getBodyPart(i), exchange);
            }
        } else if (content instanceof InputStream) {
            this.getLogger().info("add an attachment on JBI Exchange");
            exchange.addInMessageAttachment(message.getFileName(), message.getDataHandler());
        }
    }

    /**
     * Set ws addressing values : from, to, action(subject)
     * 
     * @param message
     * @param exchange
     * @throws MessagingException
     */
    protected void setProperties(Message message, final Exchange exchange)
            throws MessagingException {
        final StringBuilder s_from = new StringBuilder();
        for (final Address from : message.getFrom()) {
            s_from.append(from.toString()).append(";");
        }
        s_from.deleteCharAt(s_from.length() - 1);

        final StringBuilder s_reply = new StringBuilder();
        for (final Address reply : message.getReplyTo()) {
            s_reply.append(reply.toString()).append(";");
        }
        if (s_reply.length() > 0) {
            s_reply.deleteCharAt(s_reply.length() - 1);
        }

        StringBuilder s_to = new StringBuilder();
        for (final Address to : message.getAllRecipients()) {
            s_to.append(to.toString()).append(";");
        }
        if (s_to.length() > 0) {
            s_to = s_to.deleteCharAt(s_to.length() - 1);
        }

        final String s_subject = message.getSubject();

        this.getLogger().fine(
                "set property " + Constants.WSStar.Addressing.FROM_QNAME + " : " + s_from);
        exchange.getInMessage().setProperty(Constants.WSStar.Addressing.FROM_QNAME.toString(),
                s_from);
        if (s_reply.length() > 0) {
            this.getLogger().fine(
                    "set property " + Constants.WSStar.Addressing.REPLY_TO_QNAME + " : " + s_reply);
            exchange.getInMessage().setProperty(
                    Constants.WSStar.Addressing.REPLY_TO_QNAME.toString(), s_reply);

        }
        if (s_subject != null && s_subject.length() > 0) {
            this.getLogger().fine(
                    "set property " + Constants.WSStar.Addressing.ACTION_QNAME + " : " + s_subject);
            exchange.getInMessage().setProperty(
                    Constants.WSStar.Addressing.ACTION_QNAME.toString(), s_subject);
        }

    }

    @Override
    public void start() throws PEtALSCDKException {
        // Extract session information from SU extensions
        this.extensions = new ConfigurationExtensions(this.getConsumes().getAny());
        SessionDescriptor sessionDescriptor = null;
        try {
            sessionDescriptor = this.sessionDescriptorBuilder.build(this.extensions);

            // Create and register the listener
            MailBCExternalScheduler listener = null;
            this.t = new Timer();
            listener = new MailBCExternalScheduler(this.getLogger(), this, sessionDescriptor,
                    this.mailSessionManager);

            // Start listening for new mails
            this.t.schedule(listener, 0, Integer.parseInt(sessionDescriptor.getPeriod()));

        } catch (MissingElementException e) {
            this.getLogger().warning("Missing element in the jbi.xml : " + e.getElement());
        }

    }

    @Override
    public void stop() throws PEtALSCDKException {
        this.t.cancel();
    }
}
