/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: MailBCExternalScheduler.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.mail.listeners;

import java.util.List;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Flags.Flag;

import org.ow2.petals.bc.mail.Constants;

/**
 * This class is used to monitor a mailbox for new mails (accept() method). It
 * acts as a Listener. After new mail retrieval, the listener calls a
 * 'process(MimeMessage)' method on the MimeMessageManager, which acts as a
 * handler of MimeMessages.
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCExternalScheduler extends TimerTask {

    private final transient MailBCExternalListener externalListener;

    protected final transient Logger logger;

    protected transient MailSessionManager mailSessionManager;

    protected transient SessionDescriptor sessionDescriptor;

    /**
     * Instantiate a {@link MailBCExternalScheduler} for a specific address
     * 
     * @param externalListener
     *            the external listener for a given consumes
     * @param log
     *            the component logger
     * @param sessionDescriptor
     *            information about the mail session to open
     * @param mailSessionManager
     *            the {@link MailSessionManager}
     */
    public MailBCExternalScheduler(final Logger log, MailBCExternalListener externalListener,
            SessionDescriptor sessionDescriptor, MailSessionManager mailSessionManager) {
        super();
        this.logger = log;
        this.externalListener = externalListener;
        this.mailSessionManager = mailSessionManager;
        this.sessionDescriptor = sessionDescriptor;
    }

    public Logger getLogger() {
        return logger;
    }

    /**
     * Create mail store and folder. Retrieve new mails and process them. Mail
     * store and folder are opened (connected) and closed each time the run
     * method is called. If the processing of a message fails, it is skipped and
     * the processing of other messages continue. All processed messages are
     * deleted from the folder.
     */
    @Override
    public void run() {
        List<Message> messages = null;
        Folder folder = null;
        Store store = null;
        try {
            // Create a Session
            final Session session = this.mailSessionManager
                    .createSessionPropertiesFromDescriptor(this.sessionDescriptor);

            // Get mail store and connect to it
            store = this.mailSessionManager.getStoreAndConnect(session, this.sessionDescriptor);

            // Get mail folder and open it
            folder = this.mailSessionManager.getFolderAndOpen(store, this.sessionDescriptor);

            // receive messages
            messages = this.mailSessionManager.getNewMails(folder);
            // Process messages. If the processing of a message fails, it is
            // skipped and the processing of other messages continue
            for (final Message message : messages) {
                this.externalListener.process(message);
                message.setFlag(Flag.DELETED, true);
            }
        } catch (final MessagingException e) {
            this.getLogger().log(Level.WARNING, "error with email access : " + folder, e);
        } finally {
            if (this.sessionDescriptor.getScheme().equals(Constants.MAIL_SCHEME_POP3)
                    || this.sessionDescriptor.isDelete()) {
                this.mailSessionManager.closeFolderAndStore(folder, store, this.sessionDescriptor
                        .isDelete());
            }
        }

    }

}
