/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: MailSessionManager.java 154 28 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.mail.listeners;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.Flags.Flag;
import javax.mail.internet.MimeMessage;

import org.ow2.petals.bc.mail.Constants;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * This manager can be used to manage session with mail servers : create and
 * open sessions, send and receive mails
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailSessionManager {

    protected Logger log;

    /**
     * 
     * @param log
     *            the component logger
     */
    public MailSessionManager(Logger log) {
        super();
        this.log = log;
    }

    /**
     * Close previously opened folder and store. folder is expunged if needed
     * 
     * @param folder
     *            the folder to close
     * @param store
     *            the store to close
     * @param expunge
     */
    protected void closeFolderAndStore(Folder folder, Store store, boolean expunge) {
        try {
            if ((folder != null) && (folder.isOpen())) {
                // Delete message from the mail folder, the purge is not
                // supported
                // by POP3 protocol.
                // With POP3, the marked delete messages are expunged at the
                // disconnection.
                folder.close(expunge);
            }
        } catch (Exception ex) {
            if (folder != null) {
                log.log(Level.WARNING, "Error closing mail Folder :" + folder.toString(), ex);
            }
        }
        try {

            if ((store != null) && (store.isConnected())) {
                store.close();
            }
        } catch (Exception ex) {
            if (store != null) {
                log.log(Level.WARNING, "Error closing mail Store : " + store.toString(), ex);
            }
        }
    }

    /**
     * Create a {@link Properties} object with given properties. Then create a
     * new {@link Session} initialized with the given properties
     * 
     * @param sessionDescriptor
     *            a {@link SessionDescriptor} build from an address URI
     * @return the Mail Server {@link Session}
     */
    public Session createSessionPropertiesFromDescriptor(SessionDescriptor sessionDescriptor) {
        Properties props = new Properties();

        if (Constants.MAIL_SCHEME_SMTP.equals(sessionDescriptor.getScheme())) {
            props.put(Constants.MAIL_TRANSPORT_PROTOCOL_KEY, sessionDescriptor.getScheme());
        } else {
            props.put(Constants.MAIL_STORE_PROTOCOL_KEY, sessionDescriptor.getScheme());
        }
        props.put(Constants.MAIL_HOST_KEY, sessionDescriptor.getHostname());
        if (sessionDescriptor.getUsername() != null) {
            props.put(Constants.MAIL_USER_KEY, sessionDescriptor.getUsername());
        }
        if (sessionDescriptor.getFromAddress() != null) {
            props.put(Constants.MAIL_FROM_KEY, sessionDescriptor.getFromAddress());
        }
        if(sessionDescriptor.getHeloHost() != null) {
            props.put(Constants.MAIL_HELO_HOST, sessionDescriptor.getHeloHost());
        }

        return Session.getInstance(props);
    }

    /**
     * Retrieve a specified {@link Folder} from the given {@link Store} and open
     * it.
     * 
     * @param store
     *            the mail {@link Store}
     * @param sessionDescriptor
     *            the mail {@link SessionDescriptor}
     * @return the opened {@link Folder}
     * @throws MessagingException
     */
    public Folder getFolderAndOpen(Store store, SessionDescriptor sessionDescriptor)
            throws MessagingException {
        String msg = "";
        // Search in the specified folder of this Store
        Folder folder = null;
        folder = store.getDefaultFolder();
        if (folder == null) {
            msg = "No default folder for this store : " + store.toString();
            log.warning(msg);
            throw new MessagingException(msg);
        }
        folder = folder.getFolder(sessionDescriptor.getFolder());
        if (folder == null) {
            msg = "Invalid folder : " + sessionDescriptor.getFolder();
            log.warning(msg);
            throw new MessagingException(msg);
        }
        folder.open(Folder.READ_WRITE);
        return folder;
    }

    /**
     * Retrieve new mails into the given folder. Just unread mail are considered
     * as new mail.
     * 
     * @param folder
     *            the folder used to search for new mail
     * @return the new mail Array
     * @throws MessagingException
     */
    public List<Message> getNewMails(Folder folder) throws MessagingException {
        List<Message> messages = new ArrayList<Message>();
        Message[] lstMessage = folder.getMessages();
        // Check for new mail to process(Flag not supported with pop3 protocol)
        for (Message message : lstMessage) {
            if (!message.isSet(Flag.DELETED)) {
                messages.add(message);
            }
        }
        return messages;
    }

    /**
     * Retrieve a {@link Store} from the given Session. Then connect to it.
     * 
     * @param session
     *            the mail {@link Session}
     * @param sessionDescriptor
     *            the {@link SessionDescriptor} associated to this session
     * @return the mail {@link Store}
     * @throws MessagingException
     * @throws JBIException
     *             if error occurs during mail {@link Store} retrieval
     */
    public Store getStoreAndConnect(Session session, SessionDescriptor sessionDescriptor)
            throws MessagingException {
        Store store = null;
        // Get a Store object
        store = session.getStore(sessionDescriptor.getScheme());

        // Connect to this Store
        if (!StringHelper.isNullOrEmpty(sessionDescriptor.getUsername())
                && !StringHelper.isNullOrEmpty(sessionDescriptor.getPassword())) {
            store.connect(sessionDescriptor.getHostname(), Integer.parseInt(sessionDescriptor
                    .getPort()), sessionDescriptor.getUsername(), sessionDescriptor.getPassword());
        } else if (!StringHelper.isNullOrEmpty(sessionDescriptor.getUsername())) {

            store.connect(sessionDescriptor.getHostname(), Integer.parseInt(sessionDescriptor
                    .getPort()), sessionDescriptor.getUsername(), null);
        } else {
            store.connect(sessionDescriptor.getHostname(), Integer.parseInt(sessionDescriptor
                    .getPort()), null, null);
        }
        return store;
    }

    /**
     * Send an email based on the given MimeMessage
     * 
     * @param mimeMessage
     *            the {@link MimeMessage} to send. The resulting email will
     *            conform to the RFC822 and MIME standards
     * @param sessionDescriptor
     *            a {@link SessionDescriptor} build from an address URI
     * @param session
     *            the mail {@link Session}
     * @throws MessagingException
     *             if error occurs during message sending
     * 
     * @see Transport#send(javax.mail.Message);
     */
    public void sendMail(MimeMessage mimeMessage, SessionDescriptor sessionDescriptor,
            Session session) throws MessagingException {

        Transport transport = session.getTransport(Constants.MAIL_SCHEME_SMTP);
        if (!StringHelper.isNullOrEmpty(sessionDescriptor.getUsername())
                && !StringHelper.isNullOrEmpty(sessionDescriptor.getPassword())) {
            transport.connect(sessionDescriptor.getHostname(), Integer.parseInt(sessionDescriptor
                    .getPort()), sessionDescriptor.getUsername(), sessionDescriptor.getPassword());
        } else if (!StringHelper.isNullOrEmpty(sessionDescriptor.getUsername())) {

            transport.connect(sessionDescriptor.getHostname(), Integer.parseInt(sessionDescriptor
                    .getPort()), sessionDescriptor.getUsername(), null);
        } else {
            transport.connect(sessionDescriptor.getHostname(), Integer.parseInt(sessionDescriptor
                    .getPort()), null, null);
        }
        mimeMessage.saveChanges();
        transport.sendMessage(mimeMessage, mimeMessage.getAllRecipients());
        transport.close();

    }
}
