/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: MimeMessageManager.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.mail.listeners;

import java.util.Date;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.NormalizedMessage;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.ow2.petals.bc.mail.Constants;
import org.ow2.petals.component.framework.bc.AbstractBindingComponent;
import org.ow2.petals.component.framework.util.UtilFactory;

/**
 * This class is used to process (reading or creating) mail messages
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM Websourcing
 * 
 */
public class MimeMessageManager {

    protected AbstractBindingComponent component;

    protected Logger log;

    /**
     * 
     * @param context
     *            the {@link ComponentContext}
     * @param log
     *            the component {@link Logger}
     * @param component
     *            the component that instanciate this manager
     * @param channel
     *            the component {@link DeliveryChannel}
     */
    public MimeMessageManager(Logger log, AbstractBindingComponent component) {
        super();
        this.log = log;
        this.component = component;
    }

    /**
     * Create a {@link MimeMessage} from the given {@link NormalizedMessage},
     * including mail {@link Session} information.
     * 
     * @param session
     *            the mail {@link Session}
     * @param descriptor
     *            the {@link SessionDescriptor}
     * @param in
     *            the mapped {@link NormalizedMessage}
     * @return the created {@link MimeMessage}
     * @throws JBIException
     *             if an error occurs during {@link MimeMessage} creation
     */
    public MimeMessage mapNormalizedMessageToMimeMessage(Session session,
            SessionDescriptor descriptor, NormalizedMessage in) throws JBIException {
        this.log.finest("create email from JBI Exchange");
        final MimeMessage mimeMessage = new MimeMessage(session);
        final String sendMode = descriptor.getSendMode();
        // Set MimeMessage attributes
        this.setMimeMessageAttributes(mimeMessage, descriptor.getToAddress(), descriptor
                .getFromAddress(), descriptor.getSubject());

        final Multipart multipart = new MimeMultipart();
        if (!sendMode.equalsIgnoreCase(Constants.SEND_MODE_ATTACHMENTSONLY)) {
            // Set MimeMessage body
            this.setMimeMessageBody(multipart, in);
        }
        if (!sendMode.equalsIgnoreCase(Constants.SEND_MODE_CONTENTONLY)) {
            // Set MimeMessage attachements
            this.setMimeMessageAttachments(multipart, in);
        }
        try {
            mimeMessage.setContent(multipart);
        } catch (final javax.mail.MessagingException e) {
            final String msg = "Error setting Mime message content"
                    + "with all information collected from NormalizedMessage";
            this.log.warning(msg);
            throw new JBIException(msg, e);
        }

        return mimeMessage;
    }

    /**
     * Add {@link NormalizedMessage} attachments to {@link MimeMessage}
     * attachments. If an error occurs during the transmission of an attachment,
     * it is skipped and the process continues with other attachments.
     * 
     * @param multipart
     *            the {@link Multipart} used to handle attachments
     * @param in
     *            the {@link NormalizedMessage} handling attachments that have
     *            to be transmitted
     */
    @SuppressWarnings("unchecked")
    protected void setMimeMessageAttachments(Multipart multipart, NormalizedMessage in) {
        this.log.finest("set the email attachments");
        final Set<String> attachmentIds = in.getAttachmentNames();
        MimeBodyPart attachementBodyPart = null;
        DataHandler handler = null;
        if (attachmentIds != null) {
            for (final String id : attachmentIds) {
                handler = in.getAttachment(id);
                this.log.finest("add " + id + " as MimeMessage attachment.");
                attachementBodyPart = new MimeBodyPart();
                try {
                    attachementBodyPart.setDataHandler(handler);
                    attachementBodyPart.setFileName(id);
                    multipart.addBodyPart(attachementBodyPart);
                } catch (final javax.mail.MessagingException e) {
                    final String msg = "Error setting MimeMessage attachments " + id;
                    this.log.warning(msg);
                }
            }
        }
    }

    /**
     * Set {@link MimeMessage} recipient, date, from and subject.
     * 
     * @param mimeMessage
     *            the {@link MimeMessage} to set
     * @param internetAddress
     *            the {@link InternetAddress} of the recipient
     * @param fromInternetAddress
     *            the {@link InternetAddress} of the sender
     * @param subject
     *            the mail subject
     * @throws JBIException
     */
    protected void setMimeMessageAttributes(MimeMessage mimeMessage, String toAddress,
            String fromAddress, String subject) throws JBIException {
        this.log.finest("set the email attributes");
        try {
            // Build the destination address
            final InternetAddress toInternetAddress = new InternetAddress(toAddress);
            // Build the sender address
            final InternetAddress fromInternetAddress = new InternetAddress(fromAddress);
            mimeMessage.setRecipient(Message.RecipientType.TO, toInternetAddress);
            mimeMessage.setSentDate(new Date());
            mimeMessage.setFrom(fromInternetAddress);
            mimeMessage.setSubject(subject);
        } catch (final javax.mail.MessagingException e) {
            final String msg = "Error setting MimeMessage attributes";
            this.log.warning(msg);
            throw new JBIException(msg, e);
        }
    }

    /**
     * @see setMimeMessageBody(Multipart multipart, String content)
     * 
     * @param multipart
     *            the {@link Multipart} that will handle the
     *            {@link NormalizedMessage} content
     * @param in
     *            the {@link NormalizedMessage} that handle an xml content
     * @throws JBIException
     *             if the {@link Multipart} body setting failed
     */
    protected void setMimeMessageBody(Multipart multipart, NormalizedMessage in)
            throws JBIException {
        String content = null;
        try {
            if (in.getContent() != null) {
                content = UtilFactory.getSourceUtil().createString(in.getContent());
                this.setMimeMessageBody(multipart, content);
            }
        } catch (final Exception e) {
            throw new JBIException(e.getMessage());
        }
    }

    /**
     * Set the {@link MimeMessage} body from the given content.
     * 
     * @param multipart
     *            the {@link Multipart} that will handle the
     *            {@link NormalizedMessage} content
     * @param content
     *            the content
     * @throws MessagingException
     */
    protected void setMimeMessageBody(Multipart multipart, String content)
            throws MessagingException {
        this.log.finest("set the email body");
        final MimeBodyPart messageBodyPart = new MimeBodyPart();
        messageBodyPart.setContent(content, "text/plain");
        multipart.addBodyPart(messageBodyPart);
    }
}
