/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: SessionDescriptorBuilder.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.mail.listeners;

import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.mail.internet.InternetAddress;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.bc.mail.Constants;
import org.ow2.petals.bc.mail.MissingElementException;
import org.ow2.petals.component.framework.api.Constants.WSStar;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.bc.AbstractBindingComponent;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;

/**
 * This class is used to extract information from an SMTP URI String
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM Websourcing
 * @author Mathieu CARROLLE - EBM Websourcing
 */
public class SessionDescriptorBuilder {

    protected Logger log;

    private final AbstractBindingComponent component;

    /**
     * @param address
     *            address extracted from a "provides" node of the jbi
     *            descriptor.
     *            <p>
     *            Address conforms to the following pattern (a valid URI
     *            pattern):
     *            <p>
     *            
     *            smtp://<i>user</i>:<i>password</i>@<i>host</i>:<i>port</i>/from
     *            :<i>fromAddress</i>/to:<i>toAddress</i>.
     *            <p>
     *            <i>fromAddress</i> and <i>toAddress</i> are classical email
     *            addresses like : foo@bar.com.
     * @param log
     *            the component logger
     */
    public SessionDescriptorBuilder(Logger log, AbstractBindingComponent component) {
        super();
        this.log = log;
        this.component = component;
    }

    /**
     * Retrieve mail session properties from the given address and message
     * exchange.
     * 
     * Descriptor default values : <br/>
     * hostname : the hostname, non null and non empty<br/>
     * port : the port, default value : "25"<br/>
     * username : the username, default value : no default<br/>
     * password : the password, default value : no default<br/>
     * fromAddress : the return address, default value : no default<br/>
     * toAddress : the destination address, non null and non empty<br/>
     * period : time elapse between two new mail checkings, default value :
     * "60000" ms<br/>
     * folder : the folder to check for new mails, default value : "INBOX"<br/>
     * scheme : the access or transport protocol "smtp", "imap" or "pop3", non
     * null and non empty<br/>
     * 
     * @param extensions
     *            extensions associated to the given message exchange
     * @param exchangeWrapper
     *            the message exchange wrapper used to extract an optionnal
     *            destination address
     * @return a Map of properties extracted from the given address.
     * @throws PEtALSCDKException
     *             if address isn't a valid URI or if checking fails
     * @throws MissingElementException
     */
    public SessionDescriptor build(final ConfigurationExtensions extensions)
            throws PEtALSCDKException, MissingElementException {
        final SessionDescriptor sessionDescriptor = new SessionDescriptor();

        /*
         * Retrieve properties values from address URI
         */
        // Retrieve user and password
        sessionDescriptor.setUsername(extensions.get(Constants.USER_PATHELEMENT, null));
        sessionDescriptor.setPassword(extensions.get(Constants.PASSWORD_PATHELEMENT, null));

        // Retrieve protocol (smtp, imap, pop3)
        final String scheme = extensions.get("scheme");
        sessionDescriptor.setScheme(scheme);

        // Retrieve host and port
        sessionDescriptor.setHostname(extensions.get(Constants.HOST_PATHELEMENT));

        if (Constants.MAIL_SCHEME_POP3.equals(scheme)) {
            sessionDescriptor.setPort(extensions.get(Constants.PORT_PATHELEMENT,
                    Constants.MAIL_POP3_PORT_DEFAULT));
        } else if (Constants.MAIL_SCHEME_IMAP.equals(scheme)) {
            sessionDescriptor.setPort(extensions.get(Constants.PORT_PATHELEMENT,
                    Constants.MAIL_IMAP_PORT_DEFAULT));
        } else {
            sessionDescriptor.setPort(extensions.get(Constants.PORT_PATHELEMENT,
                    Constants.MAIL_SMTP_PORT_DEFAULT));
            String host = null;
            if((host = extensions.get(Constants.HELO_HOST_PATHELEMENT)) != null) {
                sessionDescriptor.setHeloHost(host);
            }
        }

        // process Provides or Consumes values
        if (Constants.MAIL_SCHEME_SMTP.equalsIgnoreCase(scheme)) {

            // Retrieve "from" addresses
            final String fromAddress = extensions.get(Constants.FROM_PATHELEMENT);
            if (StringHelper.isNullOrEmpty(fromAddress)) {
                throw new MissingElementException(Constants.FROM_PATHELEMENT);
            }
            sessionDescriptor.setFromAddress(fromAddress);

            // Retrieve "replyTo" addresses
            final String replyAddress = extensions.get(Constants.REPLY_PATHELEMENT);

            sessionDescriptor.setReplyAddress(replyAddress);

            final String toAddress = extensions.get(Constants.TO_PATHELEMENT);
            if (StringHelper.isNullOrEmpty(toAddress)) {
                throw new MissingElementException(Constants.TO_PATHELEMENT);
            }
            sessionDescriptor.setToAddress(toAddress);

            // Retrieve "subject"
            String subject = extensions.get(Constants.SUBJECT_PATHELEMENT);

            if (StringHelper.isNullOrEmpty(subject)) {
                subject = "petals-bc-mail";
            }
            sessionDescriptor.setSubject(subject);
            
            //retrieve send mode
            final String sendMode = extensions.get(Constants.SEND_MODE_PATHELEMENT);
            if(StringHelper.isNullOrEmpty(sendMode)){
                sessionDescriptor.setSendMode(Constants.SEND_MODE_CONTENTATTACHMENTS);
            }else {
                sessionDescriptor.setSendMode(sendMode);
            }
        } else {
            // Retrieve period parameter ("checking for new mail" interval)
            sessionDescriptor.setPeriod(extensions.get(Constants.PERIOD_QUERYELEMENT,
                    Constants.PERIOD_DEFAULT));

            // Retrieve folder parameter (the folder to search for new mails)
            sessionDescriptor.setFolder(extensions.get(Constants.FOLDER_QUERYELEMENT,
                    Constants.FOLDER_DEFAULT));

            // Retrieve the Flag to set on read messages
            final String s_expunge = extensions.get(Constants.EXPUNGE_PATHELEMENT);
            if (StringHelper.isNullOrEmpty(s_expunge)) {
                sessionDescriptor.setDelete(true);
            } else {
                sessionDescriptor.setDelete(Boolean.getBoolean(s_expunge));
            }
        }
        // Check obligatory properties
        this.checkProperties(sessionDescriptor);

        return sessionDescriptor;
    }

    /**
     * @throws MissingElementException
     * @throws PEtALSCDKException
     * 
     */
    public SessionDescriptor buildFromPayload(Exchange exchange) throws MissingElementException,
            PEtALSCDKException {
        final SessionDescriptor sessionDescriptor = this.buildTechnicalProperties();
        // find the send/mail xml element
        Document doc = null;

        try {
            doc = exchange.getInMessageContentAsDocument(true);
        } catch (final MessagingException messagingException) {
            this.log.warning(messagingException.getMessage());
        }

        final Node rootNode = doc.getFirstChild();
        if (rootNode == null || !rootNode.getLocalName().equalsIgnoreCase("mail")) {
            throw new MissingElementException("mail");
        }

        // Set the from address
        final Node fromNode = XMLUtil.findChild(rootNode, Constants.FROM_PATHELEMENT, null, false);
        try {
            new InternetAddress(fromNode.getTextContent()).validate();
            sessionDescriptor.setFromAddress(fromNode.getTextContent());
        } catch (final Exception e) {
            throw new MissingElementException(Constants.FROM_PATHELEMENT);
        }

        // Set the reply address
        final Node replyNode = XMLUtil
                .findChild(rootNode, Constants.REPLY_PATHELEMENT, null, false);
        try {
            new InternetAddress(replyNode.getTextContent()).validate();
            sessionDescriptor.setReplyAddress(replyNode.getTextContent());
        } catch (final Exception e) {
            this.log.warning("Element "+Constants.REPLY_PATHELEMENT+" is not define");
            sessionDescriptor.setReplyAddress("");
        }
        // Build the destination address
        final Node toNode = XMLUtil.findChild(rootNode, Constants.TO_PATHELEMENT, null, false);
        try {
            new InternetAddress(toNode.getTextContent()).validate();
            sessionDescriptor.setToAddress(toNode.getTextContent());
        } catch (final Exception e) {
            throw new MissingElementException(Constants.TO_PATHELEMENT);
        }

        // Set the subject
        final Node subjectNode = XMLUtil.findChild(rootNode, Constants.SUBJECT_PATHELEMENT, null,
                false);
        if (subjectNode != null) {
            sessionDescriptor.setSubject(subjectNode.getTextContent());
        } else {
            throw new MissingElementException(Constants.SUBJECT_PATHELEMENT);
        }

        // Set the body
        final Node bodyNode = XMLUtil.findChild(rootNode, Constants.BODY_PATHELEMENT, null, false);
        String body = null;
        if (bodyNode != null) {
            body = bodyNode.getTextContent();
            sessionDescriptor.setBody(body);
        } else {
            throw new MissingElementException(Constants.BODY_PATHELEMENT);
        }
        this.checkProperties(sessionDescriptor);
        return sessionDescriptor;
    }

    /***
     * Retrieve mail session properties from Normalized message properties.
     * (Used when the component supplies its own service)
     * 
     * @param exchange
     *            the message exchange used to extract properties
     * @return a map of properties
     * @throws MissingElementException
     * @throws PEtALSCDKException
     */
    public SessionDescriptor buildFromProperties(final Exchange exchange)
            throws MissingElementException, PEtALSCDKException {
        final SessionDescriptor sessionDescriptor = this.buildTechnicalProperties();
        final NormalizedMessage inNormalizedMessage = exchange.getInMessage();

        // Retrieve "from" address
        String fromAddress = (String) inNormalizedMessage.getProperty(WSStar.Addressing.FROM_QNAME
                .toString());
        if (StringHelper.isNullOrEmpty(fromAddress)) {
            fromAddress = "petals@ow2.org";
        }
        sessionDescriptor.setFromAddress(fromAddress);
        // Retrieve "to" Address
        String toAddress = null;
        try {
            if (exchange.getInMessageProperty(WSStar.ADDRESSING_KEY) != null) {
                // get the property
                final Object object = exchange.getInMessageProperty(WSStar.ADDRESSING_KEY);
                if (object instanceof Document) {
                    toAddress = WSAddressingFactory.getInstance().newWSAddressingReader()
                            .readEndpointReferenceType((Document) object).getAddress();
                }
            }
        } catch (final MessagingException e) {
            // no error to throw on this property research
        } catch (final WSAddressingException e) {
            // no error to throw on this property research
        }
        if (StringHelper.isNullOrEmpty(toAddress)) {
            toAddress = (String) inNormalizedMessage.getProperty(WSStar.Addressing.TO_QNAME
                    .toString());
        }
        if (StringHelper.isNullOrEmpty(toAddress)) {
            throw new MissingElementException(WSStar.Addressing.TO_QNAME.toString());
        }
        sessionDescriptor.setToAddress(toAddress);

        // Retrieve "replyTo" address
        String replyAddress = (String) inNormalizedMessage
                .getProperty(WSStar.Addressing.REPLY_TO_QNAME.toString());
        if (StringHelper.isNullOrEmpty(replyAddress)) {
            replyAddress = "petals@ow2.org";
        }
        sessionDescriptor.setReplyAddress(replyAddress);

        // Retrieve "subject"
        String subject = (String) inNormalizedMessage.getProperty(WSStar.Addressing.ACTION_QNAME
                .toString());
        if (StringHelper.isNullOrEmpty(subject)) {
            subject = "Mail from PEtALS";
        }
        sessionDescriptor.setSubject(subject);

        // Check mandatory properties
        this.checkProperties(sessionDescriptor);
        return sessionDescriptor;
    }

    /**
     * For the generic service, this method get the smtp properties define in
     * the jbi.xml of the component (host, port, user, password).
     * 
     * @return a Map of properties extracted from the jbi component
     */
    private SessionDescriptor buildTechnicalProperties() {

        final ConfigurationExtensions ext = this.component.getComponentExtensions();

        final SessionDescriptor sessionDescriptor = new SessionDescriptor();
        sessionDescriptor.setHostname(ext.get(Constants.HOST_PATHELEMENT));
        sessionDescriptor.setScheme(Constants.MAIL_SCHEME_SMTP);
        String port = ext.get(Constants.PORT_PATHELEMENT);
        if (StringHelper.isNullOrEmpty(port)) {
            port = Constants.MAIL_SMTP_PORT_DEFAULT;
        }
        sessionDescriptor.setPort(port);
        sessionDescriptor.setUsername(ext.get(Constants.USER_PATHELEMENT));
        sessionDescriptor.setPassword(ext.get(Constants.PASSWORD_PATHELEMENT));
        sessionDescriptor.setSendMode(Constants.SEND_MODE_CONTENTATTACHMENTS);
        return sessionDescriptor;
    }

    /**
     * Check the properties extracted from the address URI. Only hostname is
     * obligatory for all schemes. Only SMTP, POP3 and IMAP schemes are
     * supported. "toAddress" is need for SMTP scheme.
     * 
     * @param sessionDescriptor
     * @throws PEtALSCDKException
     *             if a checking step failed
     */
    protected void checkProperties(SessionDescriptor sessionDescriptor) throws PEtALSCDKException {
        String msg = "";
        if (StringHelper.isNullOrEmpty(sessionDescriptor.getHostname())) {
            msg = "The host name of the requested external " + "service must be specified";
            this.log.warning(msg);
            throw new PEtALSCDKException(msg);
        }
        if (!(Constants.MAIL_SCHEME_IMAP.equalsIgnoreCase(sessionDescriptor.getScheme())
                || Constants.MAIL_SCHEME_POP3.equalsIgnoreCase(sessionDescriptor.getScheme()) || Constants.MAIL_SCHEME_SMTP
                .equalsIgnoreCase(sessionDescriptor.getScheme()))) {
            msg = "The specified scheme isn't supported : " + sessionDescriptor.getScheme();
            this.log.warning(msg);
            throw new PEtALSCDKException(msg);
        }
    }

    /**
     * Check if the ws addressing properties are present
     * 
     * @param exchange
     * @return
     * @throws MissingElementException
     */
    public boolean checkWsaProperties(Exchange exchange) {
        // Retrieve "from" address
        final String fromAddress = (String) exchange.getInMessage().getProperty(
                WSStar.Addressing.FROM_QNAME.toString());
        // Retrieve "to" Address
        final String toAddress = (String) exchange.getInMessage().getProperty(
                WSStar.Addressing.TO_QNAME.toString());
        if (!StringHelper.isNullOrEmpty(fromAddress) || !StringHelper.isNullOrEmpty(toAddress)) {
            this.log.info("WS addressing properties found");
            return true;
        }
        return false;
    }

    public void setIOFaultOnExchange(final Exchange exchange, final String ioMessage)
            throws JBIException {
        final StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        faultMessage.append("<tns:ioFault xmlns:tns='");
        faultMessage.append(Constants.SEND_MAIL_SERVICE_NS);
        faultMessage.append("'><tns:message>");
        faultMessage.append(ioMessage);
        faultMessage.append("</tns:message></tns:ioFault>");
        final Fault fault = exchange.createFault();
        try {
            fault.setContent(UtilFactory.getSourceUtil().createSource(faultMessage.toString(),
                    CharEncoding.UTF_8));
            exchange.setFault(fault);
        } catch (final PEtALSCDKException e) {
            throw new JBIException(e.getMessage());
        }
    }

    public void setMissingElementFaultOnExchange(final Exchange exchange,
            final String missingElement) throws JBIException {
        final StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        faultMessage.append("<tns:missingElementFault xmlns:tns='");
        faultMessage.append(Constants.SEND_MAIL_SERVICE_NS);
        faultMessage.append("'><tns:element>");
        faultMessage.append(missingElement);
        faultMessage.append("</tns:element></tns:missingElementFault>");
        final Fault fault = exchange.createFault();
        try {
            fault.setContent(UtilFactory.getSourceUtil().createSource(faultMessage.toString(),
                    CharEncoding.UTF_8));
        } catch (final PEtALSCDKException e) {
            throw new JBIException(e.getMessage());
        }
        exchange.setFault(fault);
    }

}
