/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.mail.service;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.mail.BodyPart;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import org.ow2.petals.bc.mail.MissingElementException;
import org.ow2.petals.bc.mail.listeners.MailSessionManager;
import org.ow2.petals.bc.mail.listeners.MimeMessageManager;
import org.ow2.petals.bc.mail.listeners.SessionDescriptor;
import org.ow2.petals.bc.mail.listeners.SessionDescriptorBuilder;
import org.ow2.petals.component.framework.api.message.Exchange;

/**
 * 
 * @author alouis
 * @author mcarrolle
 */
public class SendMailGenericService {

    protected MailSessionManager mailSessionManager;

    protected MimeMessageManager mimeMessageManager;

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    private final Logger logger;

    /**
     * 
     * @param mailSessionManager
     * @param mimeMessageManager
     * @param sessionDescriptorBuilder
     * @param logger
     */
    public SendMailGenericService(MailSessionManager mailSessionManager,
            MimeMessageManager mimeMessageManager,
            SessionDescriptorBuilder sessionDescriptorBuilder, Logger logger) {
        super();
        this.logger = logger;
        this.mailSessionManager = mailSessionManager;
        this.mimeMessageManager = mimeMessageManager;
        this.sessionDescriptorBuilder = sessionDescriptorBuilder;
    }

    /**
     * Sending of mail by using component service
     * 
     * @param exchange
     * @throws JBIException
     */
    public void sendMailGenericService(final Exchange exchange) throws JBIException {
        if (this.sessionDescriptorBuilder.checkWsaProperties(exchange)) {
            this.sendMailServiceByProperties(exchange);
        } else {
            this.sendMailServiceByPayload(exchange);
        }
    }

    /**
     * sendMail service : retrieve from the jbi message content all the
     * information needed to send an email :<br>
     * from,reply,to,subject and body
     * 
     * @param exchange
     * @throws JBIException
     */
    protected void sendMailServiceByPayload(final Exchange exchange) throws JBIException {
        this.logger.info("JBI Exchange received - sendMail service requested");
        try {
            final SessionDescriptor sessionDescriptor = this.sessionDescriptorBuilder
                    .buildFromPayload(exchange);

            // Build the mail session
            final Session session = this.mailSessionManager
                    .createSessionPropertiesFromDescriptor(sessionDescriptor);

            final MimeMessage mimeMessage = this.mimeMessageManager
                    .mapNormalizedMessageToMimeMessage(session, sessionDescriptor, exchange
                            .getInMessage());
            // reset the body of the mail (first multipart)
            // with the 'body' element of the jbi message
            final Multipart multipart = (Multipart) mimeMessage.getContent();
            final BodyPart messageBodyPart = multipart.getBodyPart(0);
            messageBodyPart.setContent(sessionDescriptor.getBody(), "text/plain");

            // Send this mail
            if (this.logger.isLoggable(Level.FINEST)) {
                this.logger.finest("send email...");
            }
            this.mailSessionManager.sendMail(mimeMessage, sessionDescriptor, session);

            this.logger.info("email sent to " + sessionDescriptor.getToAddress());
        } catch (final MissingElementException missingException) {
            this.logger.warning("sendMail service: the element '" + missingException.getElement()
                    + "' is missing or contains a bad value");
            if (!exchange.isInOnlyPattern()) {
                this.sessionDescriptorBuilder.setMissingElementFaultOnExchange(exchange,
                        missingException.getElement());
            }

        } catch (final MessagingException me) {
            this.logger.log(Level.WARNING, "sendMail can not send the mail", me);
            if (!exchange.isInOnlyPattern()) {
                this.sessionDescriptorBuilder.setIOFaultOnExchange(exchange, me.getMessage());
            }
        } catch (IOException e) {
            exchange.setError(e);
        }

    }

    /**
     * sendMail service : retrieve from the jbi message properties all the
     * information needed to send an email :<br>
     * from,reply,to,subject and body
     * 
     * @param exchange
     * @throws JBIException
     */
    protected void sendMailServiceByProperties(final Exchange exchange) throws JBIException {
        SessionDescriptor sessionDescriptor = null;
        try {
            sessionDescriptor = this.sessionDescriptorBuilder.buildFromProperties(exchange);

            // Build the mail session
            final Session session = this.mailSessionManager
                    .createSessionPropertiesFromDescriptor(sessionDescriptor);

            // Create a mail message with the incoming jbi message
            MimeMessage mimeMessage;
            mimeMessage = this.mimeMessageManager.mapNormalizedMessageToMimeMessage(session,
                    sessionDescriptor, exchange.getInMessage());
            // Send this mail
            this.mailSessionManager.sendMail(mimeMessage, sessionDescriptor, session);
            this.logger.log(Level.INFO, "email sent to " + sessionDescriptor.getToAddress());
        } catch (final MissingElementException missingException) {
            this.logger.warning("sendMail service: the ws addressing properties '"
                    + missingException.getElement() + "' is missing or contains a bad value");
            if (exchange.isRobustInOnlyPattern()) {
                this.sessionDescriptorBuilder.setMissingElementFaultOnExchange(exchange,
                        missingException.getElement());
            }
        } catch (final MessagingException me) {
            this.logger.log(Level.WARNING, "sendMail can not send the mail", me);
            if (exchange.isRobustInOnlyPattern()) {
                this.sessionDescriptorBuilder.setIOFaultOnExchange(exchange, me.getMessage());
            }
        }
    }

}
